package com.ippon.pagination.infrastructure.primary;

import com.ippon.error.domain.Assert;
import com.ippon.pagination.domain.MyAppPage;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

@ApiModel(value = "Page", description = "Paginated content")
public class RestMyAppPage<T> {
  @ApiModelProperty(value = "Page content")
  private final List<T> content;

  @ApiModelProperty(value = "Current page (starts at 0)", required = true)
  private final int currentPage;

  @ApiModelProperty(value = "Number of elements on each page", required = true)
  private final int pageSize;

  @ApiModelProperty(value = "Total number of elements to paginate", required = true)
  private final long totalElementsCount;

  @ApiModelProperty(value = "Number of resulting pages", required = true)
  private final int pagesCount;

  private RestMyAppPage(RestMyAppPageBuilder<T> builder) {
    content = builder.content;
    currentPage = builder.currentPage;
    pageSize = builder.pageSize;
    totalElementsCount = builder.totalElementsCount;
    pagesCount = builder.pageCount;
  }

  public static <S, T> RestMyAppPage<T> from(MyAppPage<S> source, Function<S, T> mapper) {
    Assert.notNull("source", source);
    Assert.notNull("mapper", mapper);

    return new RestMyAppPageBuilder<>(source.getContent().stream().map(mapper).collect(Collectors.toList()))
      .currentPage(source.getCurrentPage())
      .pageSize(source.getPageSize())
      .totalElementsCount(source.getTotalElementsCount())
      .pageCount(source.getPageCount())
      .build();
  }

  public List<T> getContent() {
    return content;
  }

  public int getCurrentPage() {
    return currentPage;
  }

  public int getPageSize() {
    return pageSize;
  }

  public long getTotalElementsCount() {
    return totalElementsCount;
  }

  public int getPagesCount() {
    return pagesCount;
  }

  private static class RestMyAppPageBuilder<T> {
    private final List<T> content;
    private int currentPage;
    private int pageSize;
    private long totalElementsCount;
    private int pageCount;

    private RestMyAppPageBuilder(List<T> content) {
      this.content = content;
    }

    public RestMyAppPageBuilder<T> pageSize(int pageSize) {
      this.pageSize = pageSize;

      return this;
    }

    public RestMyAppPageBuilder<T> currentPage(int currentPage) {
      this.currentPage = currentPage;

      return this;
    }

    public RestMyAppPageBuilder<T> totalElementsCount(long totalElementsCount) {
      this.totalElementsCount = totalElementsCount;

      return this;
    }

    public RestMyAppPageBuilder<T> pageCount(int pageCount) {
      this.pageCount = pageCount;

      return this;
    }

    public RestMyAppPage<T> build() {
      return new RestMyAppPage<>(this);
    }
  }
}
