package com.ippon.pagination.infrastructure.secondary;

import com.ippon.error.domain.Assert;
import com.ippon.pagination.domain.MyAppPage;
import com.ippon.pagination.domain.MyAppPageable;
import java.util.function.Function;
import java.util.stream.Collectors;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

public final class MyAppPages {

  private MyAppPages() {}

  public static Pageable from(MyAppPageable pagination) {
    return from(pagination, Sort.unsorted());
  }

  public static Pageable from(MyAppPageable pagination, Sort sort) {
    Assert.notNull("pagination", pagination);
    Assert.notNull("sort", sort);

    return PageRequest.of(pagination.getPage(), pagination.getPageSize(), sort);
  }

  public static <S, T> MyAppPage<T> from(Page<S> springPage, Function<S, T> mapper) {
    Assert.notNull("springPage", springPage);
    Assert.notNull("mapper", mapper);

    return MyAppPage
      .builder(springPage.getContent().stream().map(mapper).collect(Collectors.toList()))
      .currentPage(springPage.getNumber())
      .pageSize(springPage.getSize())
      .totalElementsCount(springPage.getTotalElements())
      .build();
  }
}
