package com.ippon.error.domain;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Test;

class AssertUnitTest {

  @Test
  void shouldNotValidateNullInputs() {
    assertThatThrownBy(() -> Assert.notNull("field", null))
      .isExactlyInstanceOf(MissingMandatoryValueException.class)
      .hasMessageContaining("\"field\"");
  }

  @Test
  void shouldNotValidateNullString() {
    assertThatThrownBy(() -> Assert.notBlank("field", null))
      .isExactlyInstanceOf(MissingMandatoryValueException.class)
      .hasMessageContaining("\"field\"")
      .hasMessageContaining("(null)");
  }

  @Test
  void shouldNotValidateEmptyString() {
    assertNotBlankString("");
  }

  @Test
  void shouldNotValidateSpaceString() {
    assertNotBlankString(" ");
  }

  @Test
  void shouldNotValidateTabString() {
    assertNotBlankString("  ");
  }

  private void assertNotBlankString(String input) {
    assertThatThrownBy(() -> Assert.notBlank("field", input))
      .isExactlyInstanceOf(MissingMandatoryValueException.class)
      .hasMessageContaining("\"field\"")
      .hasMessageContaining("(blank)");
  }

  @Test
  void shouldNotValidateIntValueUnderMin() {
    assertThatThrownBy(() -> Assert.field("field", 4).min(5))
      .isExactlyInstanceOf(ValueUnderMinException.class)
      .hasMessageContaining("\"field\"")
      .hasMessageContaining("4")
      .hasMessageContaining("5");
  }

  @Test
  void shouldNotValidateIntValueOverMax() {
    assertThatThrownBy(() -> Assert.field("field", 4).max(3))
      .isExactlyInstanceOf(ValueOverMaxException.class)
      .hasMessageContaining("\"field\"")
      .hasMessageContaining("3")
      .hasMessageContaining("4");
  }
}
