package com.ippon.pagination.domain;

import static com.ippon.pagination.domain.MyAppPagesFixture.*;
import static org.assertj.core.api.Assertions.*;

import com.ippon.error.domain.MissingMandatoryValueException;
import java.util.ArrayList;
import java.util.List;
import org.junit.jupiter.api.Test;

public class MyAppPageUnitTest {

  @Test
  void shouldGetEmptySinglePageWithoutContent() {
    MyAppPage<String> page = MyAppPage.singlePage(null);

    assertEmptyPage(page);
  }

  @Test
  void shouldGetEmptySinglePageFromBuilderWithoutContent() {
    MyAppPage<?> page = MyAppPage.builder(null).build();

    assertEmptyPage(page);
  }

  private void assertEmptyPage(MyAppPage<?> page) {
    assertThat(page.getContent()).isEmpty();
    assertThat(page.getCurrentPage()).isEqualTo(0);
    assertThat(page.getPageSize()).isEqualTo(0);
    assertThat(page.getTotalElementsCount()).isEqualTo(0);
  }

  @Test
  void shouldGetSinglePage() {
    MyAppPage<String> page = MyAppPage.singlePage(List.of("test", "dummy"));

    assertSinglePage(page);
  }

  @Test
  void shouldGetSinglePageFromBuilderWithContentOnly() {
    MyAppPage<String> page = MyAppPage.builder(List.of("test", "dummy")).build();

    assertSinglePage(page);
  }

  private void assertSinglePage(MyAppPage<String> page) {
    assertThat(page.getContent()).containsExactly("test", "dummy");
    assertThat(page.getCurrentPage()).isEqualTo(0);
    assertThat(page.getPageSize()).isEqualTo(2);
    assertThat(page.getTotalElementsCount()).isEqualTo(2);
    assertThat(page.getPageCount()).isEqualTo(1);
  }

  @Test
  void shouldGetFullPage() {
    MyAppPage<String> page = fullBuilder().build();

    assertThat(page.getContent()).containsExactly("test");
    assertThat(page.getCurrentPage()).isEqualTo(2);
    assertThat(page.getPageSize()).isEqualTo(10);
    assertThat(page.getTotalElementsCount()).isEqualTo(21);
    assertThat(page.getPageCount()).isEqualTo(3);
  }

  @Test
  void shouldNotMapWithoutMapper() {
    assertThatThrownBy(() -> fullBuilder().build().map(null))
      .isExactlyInstanceOf(MissingMandatoryValueException.class)
      .hasMessageContaining("mapper");
  }

  @Test
  void shouldMapPage() {
    MyAppPage<String> page = fullBuilder().build().map(entry -> "hey");

    assertThat(page.getContent()).containsExactly("hey");
    assertThat(page.getCurrentPage()).isEqualTo(2);
    assertThat(page.getPageSize()).isEqualTo(10);
    assertThat(page.getTotalElementsCount()).isEqualTo(21);
    assertThat(page.getPageCount()).isEqualTo(3);
  }

  @Test
  void shouldNotBeLastForFirstPage() {
    assertThat(fullBuilder().currentPage(0).build().isNotLast()).isTrue();
  }

  @Test
  void shouldBeLastWithOnePage() {
    assertThat(MyAppPage.singlePage(List.of("d")).isNotLast()).isFalse();
  }

  @Test
  void shouldBeLastPageWithoutContent() {
    MyAppPage<Object> page = MyAppPage.builder(List.of()).currentPage(0).pageSize(1).totalElementsCount(0).build();

    assertThat(page.isNotLast()).isFalse();
  }

  @Test
  void shouldBeLastForLastPage() {
    assertThat(fullBuilder().currentPage(2).build().isNotLast()).isFalse();
  }

  @Test
  void shouldGetUnmodifiableContent() {
    assertThatThrownBy(() -> MyAppPage.singlePage(new ArrayList<>()).getContent().clear())
      .isExactlyInstanceOf(UnsupportedOperationException.class);
  }
}
