package com.ippon.pagination.infrastructure.primary;

import static com.ippon.pagination.domain.MyAppPagesFixture.*;
import static org.assertj.core.api.Assertions.*;

import com.ippon.error.domain.MissingMandatoryValueException;
import java.util.function.Function;
import org.junit.jupiter.api.Test;

class RestMyAppPageUnitTest {

  @Test
  void shouldNotConvertWithoutSourcePage() {
    assertThatThrownBy(() -> RestMyAppPage.from(null, source -> "test")).isExactlyInstanceOf(MissingMandatoryValueException.class);
  }

  @Test
  void shouldNotConvertWithoutMappingFunction() {
    assertThatThrownBy(() -> RestMyAppPage.from(page(), null)).isExactlyInstanceOf(MissingMandatoryValueException.class);
  }

  @Test
  void shouldMapFromDomainPage() {
    RestMyAppPage<String> page = RestMyAppPage.from(page(), Function.identity());

    assertThat(page.getContent()).containsExactly("test");
    assertThat(page.getCurrentPage()).isEqualTo(2);
    assertThat(page.getPageSize()).isEqualTo(10);
    assertThat(page.getTotalElementsCount()).isEqualTo(21);
    assertThat(page.getPagesCount()).isEqualTo(3);
  }

  @Test
  void shouldGetPageCountForPageLimit() {
    RestMyAppPage<String> page = RestMyAppPage.from(fullBuilder().totalElementsCount(3).pageSize(3).build(), Function.identity());

    assertThat(page.getPagesCount()).isEqualTo(1);
  }
}
